import org.jetbrains.kotlin.gradle.tasks.KotlinCompile

plugins {
  id("io.spring.dependency-management") version "1.0.9.RELEASE"
  java
  kotlin("jvm") version "1.3.72"
  kotlin("plugin.spring") version "1.3.72"
}

group = "eu.nk2"
version = gitTag()
java.sourceCompatibility = JavaVersion.VERSION_1_8

configurations {
  compileOnly {
    extendsFrom(configurations.annotationProcessor.get())
  }
}

repositories {
  mavenCentral()
  maven(url = "https://repo.spring.io/milestone")
  maven(url = "https://repo.spring.io/snapshot")
  jcenter()
}

dependencies {
  implementation(kotlin("stdlib-jdk8"))
  implementation("org.springframework.boot:spring-boot-starter-webflux")
  implementation("io.projectreactor:reactor-core")
  implementation("org.jetbrains.kotlinx:kotlinx-coroutines-reactor")
  implementation("io.projectreactor.kotlin:reactor-kotlin-extensions")

  implementation("com.graphql-java:graphql-java:14.0")
  implementation("com.graphql-java-kickstart:graphql-java-tools:6.0.2")
  implementation("com.graphql-java:java-dataloader:2.2.3")

  implementation("com.google.code.gson:gson:2.8.5")
  implementation("com.github.salomonbrys.kotson:kotson:2.5.0")

  testImplementation("junit", "junit", "4.12")
}

configure<JavaPluginConvention> {
  sourceCompatibility = JavaVersion.VERSION_1_8
}
tasks {
  compileKotlin {
    kotlinOptions.jvmTarget = "1.8"
  }
  compileTestKotlin {
    kotlinOptions.jvmTarget = "1.8"
  }
}

tasks.withType<KotlinCompile> {
  kotlinOptions {
    freeCompilerArgs = listOf("-Xjsr305=strict")
    jvmTarget = "1.8"
  }
}

tasks.getByName("compileJava").dependsOn("processResources")

fun gitLatestTagRev() = ProcessBuilder("git", "rev-list","--tags", "--max-count=1").start().let {
  it.waitFor(100, TimeUnit.MILLISECONDS)
  it.inputStream.bufferedReader().readLine() ?: "none"
}

fun gitTag() = ProcessBuilder("git", "describe", "--tags", gitLatestTagRev()).start().let {
  it.waitFor(100, TimeUnit.MILLISECONDS)
  it.inputStream.bufferedReader().readLine() ?: "none"
}