package eu.nk2.cache;

/*
 * Copyright (c) 2011-2017 Pivotal Software Inc, All Rights Reserved.
 * Copyright (c) 2020 nk2, All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


import java.util.List;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.function.Supplier;

import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.core.publisher.Signal;

public class CacheFlux {

  public static <KEY, VALUE> FluxCacheBuilderCacheMiss<KEY, VALUE> lookup(FluxCacheReader<KEY, VALUE> reader, KEY key) {
    return otherSupplier -> writer -> evicter ->
        Flux.defer(() ->
            MonoUtils.wrapDangerousMonoFunction(reader, key)
                .onErrorResume(error ->
                    evicter.apply(key, error)
                        .then(Mono.empty())
                )
                .switchIfEmpty(
                    otherSupplier.get()
                        .materialize()
                        .collectList()
                        .flatMap(signals ->
                            MonoUtils.wrapDangerousMonoFunction(writer, key, signals)
                                .onErrorResume(error -> evicter.apply(key, error))
                                .then(Mono.just(signals))
                        )
                )
                .flatMapIterable(Function.identity())
                .dematerialize()
        );
  }

  // ==== Support interfaces ====

  @FunctionalInterface
  public interface FluxCacheReader<KEY, VALUE> extends Function<KEY, Mono<List<Signal<VALUE>>>> {

  }

  @FunctionalInterface
  public interface FluxCacheWriter<KEY, VALUE> extends BiFunction<KEY, List<Signal<VALUE>>, Mono<Void>> {

  }

  @FunctionalInterface
  public interface FluxCacheEvicter<KEY> extends BiFunction<KEY, Throwable, Mono<Void>> {

  }

  // ==== Flux Builders ====

  public interface FluxCacheBuilderCacheMiss<KEY, VALUE> {

    default FluxCacheBuilderCacheWriter<KEY, VALUE> onCacheMissResume(Flux<VALUE> other) {
      return onCacheMissResume(() -> other);
    }

    FluxCacheBuilderCacheWriter<KEY, VALUE> onCacheMissResume(Supplier<Flux<VALUE>> otherSupplier);
  }

  public interface FluxCacheBuilderCacheWriter<KEY, VALUE> {
    FluxCacheBuilderCacheEvicter<KEY, VALUE> andWriteWith(FluxCacheWriter<? super KEY, VALUE> writer);
  }

  public interface FluxCacheBuilderCacheEvicter<KEY, VALUE> {
    Flux<VALUE> onErrorEvict(FluxCacheEvicter<? super KEY> evicter);
  }
}