package eu.nk2.cache;

/*
 * Copyright (c) 2011-2017 Pivotal Software Inc, All Rights Reserved.
 * Copyright (c) 2020 nk2, All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.function.Supplier;

import reactor.core.publisher.Mono;
import reactor.core.publisher.Signal;

public class CacheMono {

  public static <KEY, VALUE> MonoCacheBuilderCacheMiss<KEY, VALUE> lookup(MonoCacheReader<KEY, VALUE> reader, KEY key) {
    return otherSupplier -> writer -> evicter ->
        Mono.defer(() ->
            MonoUtils.wrapDangerousMonoFunction(reader, key)
                .onErrorResume(error ->
                    evicter.apply(key, error)
                        .then(Mono.empty())
                )
                .switchIfEmpty(
                    otherSupplier.get()
                        .materialize()
                        .flatMap(signal ->
                            MonoUtils.wrapDangerousMonoFunction(writer, key, signal)
                                .onErrorResume(error -> evicter.apply(key, error))
                                .then(Mono.just(signal))
                        )
                )
                .dematerialize()
        );
  }

  // ==== Support interfaces ====

  @FunctionalInterface
  public interface MonoCacheReader<KEY, VALUE> extends Function<KEY, Mono<Signal<VALUE>>> {

  }

  @FunctionalInterface
  public interface MonoCacheWriter<KEY, VALUE> extends BiFunction<KEY, Signal<VALUE>, Mono<Void>> {

  }

  @FunctionalInterface
  public interface MonoCacheEvicter<KEY> extends BiFunction<KEY, Throwable, Mono<Void>> {

  }

  // ==== Mono Builders ====

  public interface MonoCacheBuilderCacheMiss<KEY, VALUE> {
    default MonoCacheBuilderCacheWriter<KEY, VALUE> onCacheMissResume(Mono<VALUE> other) {
      return onCacheMissResume(() -> other);
    }

    MonoCacheBuilderCacheWriter<KEY, VALUE> onCacheMissResume(Supplier<Mono<VALUE>> otherSupplier);
  }

  public interface MonoCacheBuilderCacheWriter<KEY, VALUE> {
    MonoCacheBuilderCacheEvicter<KEY, VALUE> andWriteWith(MonoCacheWriter<? super KEY, VALUE> writer);
  }

  public interface MonoCacheBuilderCacheEvicter<KEY, VALUE> {
    Mono<VALUE> onErrorEvict(MonoCacheEvicter<? super KEY> evicter);
  }
}
