package eu.nk2.drm.config

import eu.nk2.drm.DrmServiceBeanPostProcessor
import eu.nk2.drm.utils.getMainClass
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.context.ConfigurableApplicationContext
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration

@Configuration
@EnableConfigurationProperties(DrmServicePropertiesHolder::class)
class DrmServiceAutoConfiguration {

    @Autowired private lateinit var configurableApplicationContext: ConfigurableApplicationContext

    @Bean fun propertiesHolder(): DrmServicePropertiesHolder =
        DrmServicePropertiesHolder()

    @Bean fun drmService(
        propertiesHolder: DrmServicePropertiesHolder
    ): DrmServiceBeanPostProcessor =
        DrmServiceBeanPostProcessor(
            url = (propertiesHolder.url ?: error("drm.url property is required to enable this server instance"))
                .replace(
                    "{package}", configurableApplicationContext.getMainClass()?.`package`?.name
                        ?: error("No class annotated with @SpringBootApplication found! Check your server instance configuration")
                ),
            connectTimeoutMillis = propertiesHolder.connectTimeoutMillis ?: error("drm.connectTimeoutMillis property is required to enable this server instance"),
            retryMaxAttempts = propertiesHolder.retryMaxAttempts ?: error("drm.retryMaxAttempts property is required to enable this server instance"),
            retryTimeoutMillis = propertiesHolder.retryTimeoutMillis ?: error("drm.retryTimeoutMillis property is required to enable this server instance")
        )
}
