import org.jetbrains.kotlin.gradle.tasks.KotlinCompile

plugins {
    id("org.springframework.boot") version "2.4.0-M1"
    id("io.spring.dependency-management") version "1.0.9.RELEASE"
    java
    kotlin("jvm") version "1.3.72"
    kotlin("plugin.spring") version "1.3.72"
    id("maven-publish")
    id("com.jfrog.artifactory") version "4.16.0"
}

group = "eu.nk2"
version = gitTag().split("-").first()
java.sourceCompatibility = JavaVersion.VERSION_1_8

configurations {
    compileOnly {
        extendsFrom(configurations.annotationProcessor.get())
    }
}

repositories {
    mavenCentral()
    maven(url = "https://repo.spring.io/milestone")
    maven(url = "https://repo.spring.io/snapshot")
    maven(url = "https://repo.nk2.eu/artifactory/gradle-release")
    jcenter()
}

dependencies {
    implementation(kotlin("stdlib-jdk8"))
    implementation("org.springframework.boot:spring-boot-starter-webflux")
    implementation("io.projectreactor:reactor-core")
    implementation("org.jetbrains.kotlinx:kotlinx-coroutines-reactor")
    implementation("io.projectreactor.kotlin:reactor-kotlin-extensions")

    implementation("com.graphql-java:graphql-java:14.0")
    implementation("com.graphql-java-kickstart:graphql-java-tools:6.0.2")
    implementation("com.graphql-java:java-dataloader:2.2.3")

    implementation("com.google.code.gson:gson:2.8.5")
    implementation("com.github.salomonbrys.kotson:kotson:2.5.0")

    implementation("eu.nk2:utils:v1.4.9")

    testImplementation("junit", "junit", "4.12")
}

configure<JavaPluginConvention> {
    sourceCompatibility = JavaVersion.VERSION_1_8
}
tasks {
    compileKotlin {
        kotlinOptions.jvmTarget = "1.8"
        kotlinOptions {
            freeCompilerArgs = listOf("-Xjsr305=strict")
        }
    }
    compileTestKotlin {
        kotlinOptions.jvmTarget = "1.8"
    }
    bootJar {
        enabled = false
    }
    jar {
        enabled = true
    }
}

tasks.getByName("compileJava").dependsOn("processResources")

fun gitLatestTagRev() = ProcessBuilder("git", "rev-list","--tags", "--max-count=1").start().let {
    it.waitFor(100, TimeUnit.MILLISECONDS)
    it.inputStream.bufferedReader().readLine() ?: "none"
}

fun gitTag() = ProcessBuilder("git", "describe", "--tags", gitLatestTagRev()).start().let {
    it.waitFor(100, TimeUnit.MILLISECONDS)
    it.inputStream.bufferedReader().readLine() ?: "none"
}

val sourcesJar by tasks.creating(Jar::class) {
    dependsOn(JavaPlugin.CLASSES_TASK_NAME)
    classifier = "sources"
    from(sourceSets["main"].allSource)
}

publishing {
    publications {
        create<MavenPublication>("mavenJava") {
            groupId = project.group.toString()
            version = project.version.toString()
            artifactId = project.name

            from(components["java"])
            artifact(sourcesJar)
            artifact("$buildDir/libs/${project.name}-${project.version}.jar")
        }
    }
}

artifactory {
    setContextUrl(project.findProperty("artifactory_contextUrl"))
    publish(delegateClosureOf<org.jfrog.gradle.plugin.artifactory.dsl.PublisherConfig> {
        repository(delegateClosureOf<groovy.lang.GroovyObject> {
            setProperty("repoKey", "gradle-release-local")
            setProperty("username", project.findProperty("artifactory_user") ?: "nouser")
            setProperty("password", project.findProperty("artifactory_password") ?: "nopass")
            setProperty("maven", true)
        })
        defaults(delegateClosureOf<groovy.lang.GroovyObject> {
            invokeMethod("publications", "mavenJava")
//            invokeMethod("publishConfigs", "archives")
            setProperty("publishArtifacts", true)
            setProperty("publishPom", true)
            setProperty("publishIvy", false)
        })
    })
    resolve(delegateClosureOf<org.jfrog.gradle.plugin.artifactory.dsl.ResolverConfig> {
        repository(delegateClosureOf<groovy.lang.GroovyObject> {
            setProperty("repoKey", "gradle-release")
            setProperty("username", project.findProperty("artifactory_user") ?: "nouser")
            setProperty("password", project.findProperty("artifactory_password") ?: "nopass")
            setProperty("maven", true)
        })
    })
}