package eu.nk2.graphql

import com.google.gson.annotations.Expose
import graphql.ErrorClassification
import graphql.language.SourceLocation
import org.springframework.http.HttpStatus
import org.springframework.web.bind.annotation.ResponseStatus

@ResponseStatus(value = HttpStatus.NOT_FOUND)
class ResourceNotFoundException(type: String, key: String, value: Any?, @Expose(serialize = false) private val prettyMessage: String = "Required $key of $type not found: $value"): BaseGraphQlException(HttpStatus.NOT_FOUND, prettyMessage) {
    override fun errorMessage(): String = prettyMessage
    override fun getErrorType(): ErrorClassification? = null
    override fun getLocations(): MutableList<SourceLocation>? = null
}

@ResponseStatus(value = HttpStatus.CONFLICT)
class ResourceDuplicateException(type: String, key: String, value: Any?, @Expose(serialize = false) private val prettyMessage: String = "$key of $type is already exists: $value"): BaseGraphQlException(HttpStatus.CONFLICT, prettyMessage) {
    override fun errorMessage(): String = prettyMessage
    override fun getErrorType(): ErrorClassification? = null
    override fun getLocations(): MutableList<SourceLocation>? = null
}

@ResponseStatus(value = HttpStatus.BAD_REQUEST)
class ResourceTypeNotValidException(type: String, requestedType: String, requiredType: String, private val prettyMessage: String = "Operation is permitted for $type $requestedType, $requiredType is required"): BaseGraphQlException(HttpStatus.BAD_REQUEST, prettyMessage) {
    override fun errorMessage(): String = prettyMessage
    override fun getErrorType(): ErrorClassification? = null
    override fun getLocations(): MutableList<SourceLocation>? = null
}

@ResponseStatus(value = HttpStatus.FORBIDDEN)
class ResourceDataMismatchException(type: String, key: String, originalValue: Any?, value: Any?, private val prettyMessage: String = "Required $key of $type is wrong: $value was provided instead of $originalValue"): BaseGraphQlException(HttpStatus.FORBIDDEN, prettyMessage) {
    override fun errorMessage(): String = prettyMessage
    override fun getErrorType(): ErrorClassification? = null
    override fun getLocations(): MutableList<SourceLocation>? = null
}

@ResponseStatus(value = HttpStatus.INTERNAL_SERVER_ERROR)
class ResourceInvalidException(type: String, value: Any?, private val prettyMessage: String = "Resource of type $type is invalid: $value"): BaseGraphQlException(HttpStatus.INTERNAL_SERVER_ERROR, prettyMessage) {
    override fun errorMessage(): String = prettyMessage
    override fun getErrorType(): ErrorClassification? = null
    override fun getLocations(): MutableList<SourceLocation>? = null
}
