package eu.nk2.graphql

import graphql.kickstart.tools.GraphQLResolver
import graphql.kickstart.tools.SchemaParser
import graphql.kickstart.tools.SchemaParserOptions
import graphql.schema.GraphQLEnumType
import graphql.schema.GraphQLScalarType
import graphql.schema.GraphQLSchema
import kotlinx.coroutines.ExperimentalCoroutinesApi
import reactor.core.publisher.Mono

@ExperimentalCoroutinesApi
fun buildGraphQlSchema(
  schemaFile: String,
  resolvers: Array<GraphQLResolver<*>>,
  scalars: Array<GraphQLScalarType> = arrayOf(),
  dictionary: Map<String, Class<*>> = mapOf()
): GraphQLSchema {
  return SchemaParser.newParser()
    .file(schemaFile)
    .resolvers(*resolvers)
    .scalars(*scalars)
    .dictionary(dictionary)
    .options(
      SchemaParserOptions.newOptions()
        .genericWrappers(SchemaParserOptions.GenericWrapper(Mono::class.java, 0))
        .build()
    )
    .build()
    .makeExecutableSchema()
}
