package eu.nk2.graphql

import com.google.gson.Gson
import eu.nk2.graphql.data.GraphQlContext
import graphql.ExecutionInput
import graphql.ExecutionResult
import graphql.schema.GraphQLSchema
import org.springframework.http.MediaType
import reactor.core.publisher.Mono
import reactor.core.publisher.Mono.*
import java.net.URLDecoder

val GraphQlMediaType = MediaType.parseMediaType("application/GraphQL")
const val GraphQlMediaTypeValue = "application/GraphQL"

data class GraphQlParameters(
  val query: String,
  val operationName: String? = null,
  val variables: Map<String, Any>? = null
)

fun <CTX> executeGraphQlQuery(graphQlContext: GraphQlContext<CTX>, graphQlParameters: GraphQlParameters): Mono<ExecutionResult> {
  val executionInput = ExecutionInput.newExecutionInput()
    .query(graphQlParameters.query)
    .operationName(graphQlParameters.operationName)
    .variables(graphQlParameters.variables ?: mapOf())

  return graphQlContext.executeQuery(executionInput)
}

fun getGraphQlVariables(gson: Gson, variables: String?): Map<String, Any>? =
  readJsonMap(
    gson = gson,
    variables = variables?.let { URLDecoder.decode(it, "UTF-8") }
  )
