package eu.nk2.graphql.batch

import eu.nk2.utils.asyncMap
import org.dataloader.DataLoader
import org.dataloader.DataLoaderOptions
import org.dataloader.DataLoaderRegistry
import org.dataloader.MappedBatchLoader
import reactor.core.publisher.Flux

interface BatchedService<T: Any, ID: Any> {
    val dataLoaderKey: String
    fun provideIdByValue(keys: List<ID>, value: T): ID
    fun findAllByIds(ids: List<ID>): Flux<T>
}

fun <T: Any, ID: Any> BatchedService<T, ID>.makeBatchLoader(): MappedBatchLoader<ID, T> =
    MappedBatchLoader { keys ->
        val keysList = keys.toList()
        this.findAllByIds(keysList)
            .collectList()
            .asyncMap { results ->
                results.map { this.provideIdByValue(keysList, it) to it }
                    .toMap()
            }
            .toFuture()
    }
