package eu.nk2.graphql.scalar

import com.github.salomonbrys.kotson.fromJson
import com.google.gson.Gson
import graphql.language.StringValue
import graphql.schema.*

class MapScalar(private val gson: Gson): GraphQLScalarType(
  "Map",
  "Map<String, Any> scalar",
  object : Coercing<Map<String, Any>?, String?> {
    @Throws(CoercingSerializeException::class)
    override fun serialize(input: Any): String? =
      gson.toJson(when (input) {
        is Map<*, *> -> (input as? Map<String, Any>) ?: throw CoercingSerializeException("Expected a 'Map<String, Any>' but was '${typeName(input)}'.")
        else -> throw CoercingSerializeException("Expected a 'Map<String, Any>' but was '${typeName(input)}'.")
      })

    @Throws(CoercingParseValueException::class)
    override fun parseValue(input: Any): Map<String, Any>? =
      when (input) {
        is Map<*, *> -> (input as? Map<String, Any>) ?: throw CoercingSerializeException("Expected a 'Map<String, Any>' but was '${typeName(input)}'.")
        is String -> gson.fromJson(input)
        else -> throw CoercingSerializeException("Expected a 'Map<String, Any>' but was '${typeName(input)}'.")
      }

    @Throws(CoercingParseLiteralException::class)
    override fun parseLiteral(input: Any): Map<String, Any>? {
      if (input !is StringValue)
        throw CoercingParseLiteralException("Expected AST type 'StringValue' but was '${typeName(input)}'.")

      return gson.fromJson(input.value)
    }

  }
) {
  companion object {
    private fun typeName(input: Any?): String? {
      return if (input == null) {
        "null"
      } else input.javaClass.simpleName
    }
  }
}
