package eu.nk2.messaging

import com.google.gson.Gson
import org.springframework.http.HttpHeaders
import org.springframework.http.MediaType
import org.springframework.web.reactive.function.BodyInserters
import org.springframework.web.reactive.function.client.WebClient
import org.springframework.web.util.UriBuilder
import reactor.core.publisher.Mono

class Sender(
    private val token: String,
    private val webClient: WebClient,
    private val gson: Gson
) {
    fun send(message: Message) =
        webClient.post()
            .uri(uriBuilder)
            .headers { headersBuilder(token, it) }
            .contentType(MediaType.APPLICATION_JSON)
            .body(BodyInserters.fromValue(gson.toJson(message)))
            .retrieve()
            .bodyToMono(Map::class.java)
            .onErrorResume { Mono.just(mapOf("error" to it)) }

    companion object {
        private val uriBuilder = { it: UriBuilder ->
            it.scheme("https")
                .host("fcm.googleapis.com")
                .path("/fcm/send")
                .build()
        }
        private val headersBuilder = { token: String, it: HttpHeaders ->
            it.set("Authorization", "key=$token")
        }
    }
}
