package eu.nk2.ref

import java.lang.reflect.Field
import java.lang.reflect.Modifier
import java.util.*


fun Class<*>.findFields(vararg annotationClasses: Class<out Annotation?>): Set<Field> {
  val set: MutableSet<Field> = HashSet<Field>()
  var currentClass: Class<*>? = this
  while (currentClass != null) {
    for (field in currentClass.declaredFields)
      if (annotationClasses.any { field.isAnnotationPresent(it) })
        set.add(field)

    currentClass = currentClass.superclass
  }
  return set
}

inline fun <reified T> Field.getUsingFieldOrGetter(value: Any): T? =
  if(Modifier.isPrivate(modifiers) || Modifier.isProtected(modifiers)) value.javaClass.getMethod("get${name.capitalize()}").invoke(value) as T?
  else get(value) as T?
