package eu.nk2.utils;

import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import org.springframework.core.io.Resource
import org.springframework.http.MediaType
import org.springframework.web.reactive.function.BodyInserters
import org.springframework.web.reactive.function.server.ServerRequest
import org.springframework.web.reactive.function.server.ServerResponse
import org.springframework.web.reactive.function.server.bodyToMono
import reactor.core.publisher.Mono


fun ServerRequest.contentTypeIs(mediaType: MediaType) =
    this.headers().contentType().filter { it.isCompatibleWith(mediaType) }.isPresent

fun <T> ServerRequest.withBody(mapFun: (String) -> T): Mono<T> =
    this.bodyToMono<String>().flatMap { Mono.just(mapFun(it)) }

inline fun <reified T> readJson(gson: Gson, value: String): T = gson.fromJson(value, T::class.java)
fun readJsonMap(gson: Gson, variables: String?): Map<String, Any>? = variables?.let { gson.fromJson(it, TypeToken.getParameterized(Map::class.java, String::class.java, Any::class.java).type) }

fun serveStatic(resource: Resource): (ServerRequest) -> Mono<ServerResponse> =
    { ServerResponse.ok().body(BodyInserters.fromResource(resource)) }
