package eu.nk2.utils

import reactor.core.publisher.Flux
import reactor.core.publisher.Mono
import reactor.core.scheduler.Schedulers
import java.util.*

fun <T> Mono<T>.errorIfEmpty(throwable: Throwable): Mono<T> =
    this.switchIfEmpty(Mono.defer { Mono.error<T>(throwable) })

fun <T, R> Mono<T>.flatMapMonos(monosIterable: Iterable<Mono<R>>) =
    this.flatMapIterable { monosIterable }
        .flatMap { it }

fun <T, R> Mono<out Iterable<T>>.flatMapMonoFunctions(mapper: (T) -> Mono<R>) =
    this.flatMapIterable { it }
        .flatMap { mapper(it) }

fun <T, R> Mono<T>.mapWith(other: R) =
    this.map { it to other }

fun <T, R, U> Mono<T>.mapWith(other1: R, other2: U) =
    this.map { Triple(it, other1, other2) }

fun <T, R> Mono<T>.mapWith(other: Function1<T, R>) =
    this.asyncMap { it to other(it) }

fun <T, R, U> Mono<Pair<T, R>>.mapWithPair(other: U) =
    this.map { (first, second) -> Triple(first, second, other) }

fun <T, R> Flux<T>.mapWith(other: R) =
    this.map { it to other }

fun <T, R> Flux<T>.mapWith(other: Function1<T, R>) =
    this.asyncMap { it to other(it) }

fun <T, R, U> Flux<Pair<T, R>>.mapWithPair(other: U) =
    this.map { (first, second) -> Triple(first, second, other) }

fun <T> Mono<T>.wrapOptional(): Mono<Optional<T>> =
    this.map { Optional.of(it) }
        .switchIfEmpty(Mono.fromCallable { Optional.empty<T>() })

fun <T> Optional<T>.orNull(): T? =
    if(this.isPresent) this.get()
    else null

inline fun <T, R> Flux<T>.flatMapFilterNotEmpty(crossinline mapper: (T) -> (Mono<R>)) =
    this.flatMap { mapper(it).materialize() }
        .filter { it.hasValue() }
        .dematerialize<R>()

fun <T, R> Mono<T>.asyncMap(mapper: (T) -> R): Mono<R> =
  this.flatMap {
      Mono.fromCallable {
          mapper(it)
      }
      .subscribeOn(Schedulers.boundedElastic())
  }

fun <T, R> Flux<T>.asyncMap(mapper: (T) -> R): Flux<R> =
  this.flatMap {
      Mono.fromCallable {
        mapper(it)
      }
      .subscribeOn(Schedulers.boundedElastic())
  }

fun <T, R> Flux<T>.firstMapWith(other: R) =
    this.map { other to it }

fun <T, R> Flux<T>.firstMapWith(other: Function1<T, R>) =
    this.asyncMap { other(it) to it }

fun <T, R, U> Flux<Pair<T, R>>.firstMapWithPair(other: U) =
    this.map { (first, second) -> Triple(other, first, second) }

fun <T, R> Mono<T>.firstMapWith(other: R) =
    this.map { other to it }

fun <T, R, U> Mono<T>.firstMapWith(other1: R, other2: U) =
    this.map { Triple(other1, other2, it) }

fun <T, R> Mono<T>.firstMapWith(other: Function1<T, R>) =
    this.asyncMap { other(it) to it }

fun <T, R, U> Mono<Pair<T, R>>.firstMapWithPair(other: U) =
    this.map { (first, second) -> Triple(other, first, second) }


