package eu.nk2.utils

private val firstLetters = mapOf(
    "а" to "a",
    "б" to "b",
    "в" to "v",
    "д" to "d",
    "з" to "z",
    "й" to "y",
    "к" to "k",
    "л" to "l",
    "м" to "m",
    "н" to "n",
    "о" to "o",
    "п" to "p",
    "р" to "r",
    "с" to "s",
    "т" to "t",
    "у" to "u",
    "ф" to "f",
    "ь" to "",
    "г" to "g",
    "и" to "i",
    "ъ" to "",
    "ы" to "i",
    "э" to "e"
)

private val initialDigraphs = mapOf("е" to "ye")
private val regularDigraphs = mapOf(
    "ё" to "yo",
    "ж" to "zh",
    "х" to "kh",
    "ц" to "ts",
    "ч" to "ch",
    "ш" to "sh",
    "щ" to "shch",
    "ю" to "yu",
    "я" to "ya"
)
private val firstDigraphs = regularDigraphs + initialDigraphs
private val firstAssociations = firstLetters + firstDigraphs
private val nonFirstLetters =  firstLetters + mapOf("й" to "i", "е" to "e")

// digraphs appearing only in non-initial positions
private val nonInitialDigraphs = mapOf<String, String>()
private val nonFirstDigraphs = regularDigraphs + nonInitialDigraphs
private val nonFirstAssociations = nonFirstLetters + nonFirstDigraphs

fun String.cyrillicToTranslit(spaceReplacement: Char? = null): String {
    var newStr = ""
    var isWordBoundary = false

    for(i in this.indices) {
        val isUpperCaseOrWhatever = this[i] == this[i].toUpperCase()
        val strLowerCase = this[i].toLowerCase()
        if (strLowerCase == ' ') {
            newStr += spaceReplacement ?: ' '
            isWordBoundary = true
            continue
        }

        val newLetter = when {
            i == 0 || isWordBoundary -> {
                isWordBoundary = false
                firstAssociations[strLowerCase.toString()]
            }
            else -> nonFirstAssociations[strLowerCase.toString()]
        }

        when {
            newLetter == null -> newStr += if(isUpperCaseOrWhatever) strLowerCase.toUpperCase() else strLowerCase
            isUpperCaseOrWhatever -> newStr += if(newLetter.isNotEmpty()) newLetter[0].toUpperCase() + newLetter.substring(1) else newLetter.toUpperCase()
            else -> newStr += newLetter
        }

    }

    return newStr
}
