import eu.nk2.utils.asyncMap
import org.hamcrest.Matchers
import org.junit.Assert.assertThat
import org.junit.Test
import reactor.core.publisher.Flux
import reactor.core.publisher.Mono
import reactor.kotlin.core.publisher.toMono

class MonoUtilsTest {
  @Test fun monoAsyncMap_single() {
    assertThat(
      "Mono.asyncMap works with single call",
      5.toMono()
        .asyncMap { it + 10 }
        .block()!!,
      Matchers.equalTo(15)
    )
  }

  @Test fun monoAsyncMap_multi() {
    assertThat(
      "Mono.asyncMap works with multi call",
      5.toMono()
        .asyncMap { it + 10 }
        .asyncMap { it * 10 }
        .asyncMap { it - 10 }
        .block()!!,
      Matchers.equalTo(140)
    )
  }

  @Test fun monoAsyncMap_multiMixed() {
    assertThat(
      "Mono.asyncMap works with multi call mixed with other ops",
      5.toMono()
        .asyncMap {
          Thread.sleep(50)
          it + 10
        }
        .map { "$it!" }
        .flatMap { Mono.empty<String>() }
        .switchIfEmpty("A".toMono())
        .asyncMap { it.repeat(10) }
        .block()!!,
      Matchers.equalTo("AAAAAAAAAA")
    )
  }

  @Test fun fluxAsyncMap_single() {
    assertThat(
      "Flux.asyncMap works with single call",
      Flux.fromArray(arrayOf(1, 2, 3, 4, 5))
        .asyncMap { it + 10 }
        .collectList()
        .block()!!,
      Matchers.equalTo(
        listOf(11, 12, 13, 14, 15)
      )
    )
  }

  @Test fun fluxAsyncMap_multi() {
    assertThat(
      "Flux.asyncMap works with multi call",
      Flux.fromArray(arrayOf(1, 2, 3, 4, 5))
        .asyncMap { it + 10 }
        .asyncMap { it * 10 }
        .asyncMap { it - 10 }
        .collectList()
        .block()!!,
      Matchers.equalTo(
        listOf(100, 110, 120, 130, 140)
      )
    )
  }

  @Test fun fluxAsyncMap_multiMixed() {
    assertThat(
      "Flux.asyncMap works with multi call mixed with other ops",
      Flux.fromArray(arrayOf(1, 2, 3, 4, 5))
        .asyncMap {
          Thread.sleep(50)
          it + 10
        }
        .groupBy { it % 2 == 0 }
        .flatMap { group ->
          group.collectList()
            .map { group.key()!! to it }
        }
        .asyncMap { (k, v) -> "$k: ${v.joinToString(separator = ",") { it.toString() }}!" }
        .collectList()
        .block()!!,
      Matchers.equalTo(listOf("false: 11,13,15!", "true: 12,14!"))
    )
  }
}